/*----------------------------------------------------------------------------*/
// File: geo_util.c   USGS Navigation, Geodesy utilities
// Author: John Gann
// Start Date: 9 April 1992
/*----------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <time.h>
#include <fcntl.h>
#include <io.h>

#include <GRconst.h>
#include <GRports.h>
#include <GRextrn.h>
#include <menuet.h>

#include "navdefs.h"
#include "navtypes.h"
#include "utmlib.h"
#include "nav_util.h"           // Prototypes
#include "lines.h"              // Prototypes
#include "graph.h"              // Prototypes
#include "mnutils.h"           // Prototypes
#include "geo_util.h"           // Prototypes

static t_pos lastFrom;
static t_pos lastTo;
static double lastRange = 0.0;
static double lastBearing = 0.0;

static char * CANT_OPEN_MSG = "Can't Open";
static char * WARNING_MSG =   "Warning";

/*----------------------------------------------------------------------------*/
// do Range and Bearing form
/*----------------------------------------------------------------------------*/
int doRangeBearingFm ( ob_desc * ob )
{
int retv;
fm_desc * fm;
char * frf = "r_and_b.frf";
ftag * fromLatTag;
ftag * fromLonTag;
ftag * toLatTag;
ftag * toLonTag;
ftag * rangeTag;
ftag * bearingTag;
char buf[80];

// Load the FRF file 
if ( ( fm = loadFRF ( frf ) ) == NULL ) 
	{
  return FALSE;
	}

OpenForm ( fm, 10, ob->area.Ymin ); 

// set up form
fromLatTag = (ftag *) fm->ob_lst[2].tag;
fromLonTag = (ftag *) fm->ob_lst[3].tag;
toLatTag   = (ftag *) fm->ob_lst[4].tag;
toLonTag   = (ftag *) fm->ob_lst[5].tag;
rangeTag   = (ftag *) fm->ob_lst[6].tag;
bearingTag = (ftag *) fm->ob_lst[7].tag;

// update from position
PositionString(fromLonTag->fld, 
               fromLatTag->fld, 
               buf, 
               lastFrom.lon, 
               lastFrom.lat, 
               0.0, 0);

// update to position
PositionString(toLonTag->fld, 
               toLatTag->fld, 
               buf, 
               lastTo.lon, 
               lastTo.lat, 
               0.0, 0);

// clear range and bearing
strset(rangeTag->fld, '\0');
strset(bearingTag->fld, '\0');

DrawTaggedOb(&fm->ob_lst[2]); 
DrawTaggedOb(&fm->ob_lst[3]); 
DrawTaggedOb(&fm->ob_lst[4]); 
DrawTaggedOb(&fm->ob_lst[5]); 
DrawTaggedOb(&fm->ob_lst[6]); 
DrawTaggedOb(&fm->ob_lst[7]); 

SetFldExitTask( PositionExitTask );

do 
	{
	retv = ProcessObList ( fm->ob_lst, fm->ob_cnt, 0 );
	switch ( retv ) 
		{
		case CALCULATE:
			// get the field values in to radians
			if ((retv = calculateRangeBearing(fm, 2)) != SUCCESS)
				{
				NotifyUser(-1, -1, WARNING_MSG, "Can't compute");
				retv = 99;
				}
			break;

		case DONE:
			break;

		default : 
			break;
		}
	} while (retv != DONE && retv != NOT_SET);

CloseForm   ( fm );
DestroyFRFForm ( fm );

return retv;
}



/*----------------------------------------------------------------------------*/
// Suck positions from a two position form
/*----------------------------------------------------------------------------*/
int suckPositions(fm_desc * fm, int idx, t_pos * pos1, t_pos * pos2)
{
ftag * slatTag = (ftag *) fm->ob_lst[idx+0].tag;
ftag * slonTag = (ftag *) fm->ob_lst[idx+1].tag;
ftag * elatTag = (ftag *) fm->ob_lst[idx+2].tag;
ftag * elonTag = (ftag *) fm->ob_lst[idx+3].tag;

if (!PositionStrToRads(slatTag->fld, &pos1->lat, LAT))
	return BAD_LATITUDE;

if (!PositionStrToRads(slonTag->fld, &pos1->lon, LON))
	return BAD_LONGITUDE;

if (!PositionStrToRads(elatTag->fld, &pos2->lat, LAT))
	return BAD_LATITUDE;

if (!PositionStrToRads(elonTag->fld, &pos2->lon, LON))
	return BAD_LONGITUDE;

return SUCCESS;
}



/*----------------------------------------------------------------------------*/
// Compute the range and bearing of the current from and to points
/*----------------------------------------------------------------------------*/
int calculateRangeBearing(fm_desc * fm, int idx)
{
ftag * rngTag  = (ftag *) fm->ob_lst[idx+4].tag;
ftag * brgTag  = (ftag *) fm->ob_lst[idx+5].tag;
double back_azm;
t_ellips * ellipse = query_ellipsoid();   // ellipsoid for arclength comps
int retv;

if ((retv = suckPositions(fm, idx, &lastFrom, &lastTo)) != SUCCESS)
	return retv;

if (lastFrom.lat == 0.0 || lastFrom.lon == 0.0 || 
      lastTo.lat == 0.0 || lastTo.lon == 0.0)
	return BAD_POSITION;

// compute the range and bearing
if (arclen(lastFrom.lat, lastFrom.lon, lastTo.lat, lastTo.lon, 
					&lastRange, &lastBearing, &back_azm, 
					ellipse->a, ellipse->b) == SUCCESS)
	{
	sprintf(rngTag->fld, "%7.1lf", lastRange);
	sprintf(brgTag->fld, "%5.1lf", lastBearing * RAD_TO_DEG);
 	DrawTaggedOb(&fm->ob_lst[idx+4]); 
	DrawTaggedOb(&fm->ob_lst[idx+5]); 
	}

return SUCCESS;
}




/*----------------------------------------------------------------------------*/
// do Inverse form
/*----------------------------------------------------------------------------*/
int doInverseFm ( ob_desc * ob )
{
int retv;
fm_desc * fm;
char * frf = "inverse.frf";
ftag * fromLatTag;
ftag * fromLonTag;
ftag * toLatTag;
ftag * toLonTag;
ftag * rangeTag;
ftag * bearingTag;
char buf[80];

// Load the FRF file 
if ( ( fm = loadFRF ( frf ) ) == NULL ) 
	{
  return FALSE;
	}

fm->ob_lst[6].flags &= ~IS_EDITABLE;
fm->ob_lst[7].flags &= ~IS_EDITABLE;

OpenForm ( fm, 10, ob->area.Ymin ); 

// set up form
fromLatTag = (ftag *) fm->ob_lst[2].tag;
fromLonTag = (ftag *) fm->ob_lst[3].tag;
rangeTag   = (ftag *) fm->ob_lst[4].tag;
bearingTag = (ftag *) fm->ob_lst[5].tag;
toLatTag   = (ftag *) fm->ob_lst[6].tag;
toLonTag   = (ftag *) fm->ob_lst[7].tag;

// update from position
PositionString(fromLonTag->fld, 
               fromLatTag->fld, 
               buf, 
               lastFrom.lon, 
               lastFrom.lat, 
               0.0, 0);

// update range and bearing
sprintf(rangeTag->fld, "%.1lf", lastRange);
sprintf(bearingTag->fld, "%.1lf", lastBearing * RAD_TO_DEG);

// clear new lat and lon
strset(toLatTag->fld, '\0');
strset(toLonTag->fld, '\0');

DrawTaggedOb(&fm->ob_lst[2]); 
DrawTaggedOb(&fm->ob_lst[3]); 
DrawTaggedOb(&fm->ob_lst[4]); 
DrawTaggedOb(&fm->ob_lst[5]); 
DrawTaggedOb(&fm->ob_lst[6]); 
DrawTaggedOb(&fm->ob_lst[7]); 

SetFldExitTask( PositionExitTask );

do 
	{
	retv = ProcessObList ( fm->ob_lst, fm->ob_cnt, 0 );
	switch ( retv ) 
		{
		case CALCULATE:
			// get the field values in to radians
			if ((retv = calculateInverse(fm, 2)) != SUCCESS)
				{
				NotifyUser(-1, -1, WARNING_MSG, "Can't compute");
				retv = 99;
				}
			break;

		case DONE:
			break;

		default : 
			break;
		}
	} while (retv != DONE && retv != NOT_SET);

CloseForm   ( fm );
DestroyFRFForm ( fm );

return retv;
}


/*----------------------------------------------------------------------------*/
// Compute the inverse 
/*----------------------------------------------------------------------------*/
int calculateInverse(fm_desc * fm, int idx)
{
ftag * slatTag = (ftag *) fm->ob_lst[idx+0].tag;
ftag * slonTag = (ftag *) fm->ob_lst[idx+1].tag;
ftag * rngTag  = (ftag *) fm->ob_lst[idx+2].tag;
ftag * brgTag  = (ftag *) fm->ob_lst[idx+3].tag;
ftag * elatTag = (ftag *) fm->ob_lst[idx+4].tag;
ftag * elonTag = (ftag *) fm->ob_lst[idx+5].tag;
t_ellips * ellipse = query_ellipsoid();   // ellipsoid for arclength comps
int retv;
char buf[80];

if (!PositionStrToRads(slatTag->fld, &lastFrom.lat, LAT))
	return BAD_LATITUDE;

if (!PositionStrToRads(slonTag->fld, &lastFrom.lon, LON))
	return BAD_LONGITUDE;

lastRange = atof(rngTag->fld);
lastBearing = DEG_TO_RAD * atof(brgTag->fld);

// compute inverse
if ((retv = proj(lastFrom.lat, lastFrom.lon, lastRange, lastBearing, 
                &lastTo.lat, &lastTo.lon, 
                ellipse->a, ellipse->b)) == SUCCESS)
	{
	PositionString(elonTag->fld, 
                 elatTag->fld, 
                 buf, 
                 lastTo.lon, 
                 lastTo.lat, 
                 0.0, 0);

 	DrawTaggedOb(&fm->ob_lst[idx+4]); 
	DrawTaggedOb(&fm->ob_lst[idx+5]); 
	}
else
	{
	return retv;
	}

return SUCCESS;
}




/*----------------------------------------------------------------------------*/
// do Default UTM Zone form
/*----------------------------------------------------------------------------*/
int doUtmZone ( ob_desc * ob )
{
int retv;
fm_desc * fm;
char * frf = "utm_zone.frf";
ftag * ZoneLtrTag;
ftag * ZoneNumTag;
// char buf[80];
t_setup * config = QuerySetup();

// Load the FRF file 
if ( ( fm = loadFRF ( frf ) ) == NULL ) 
	{
  return FALSE;
	}

// set up form
ZoneNumTag = (ftag *) fm->ob_lst[2].tag;
ZoneLtrTag = (ftag *) fm->ob_lst[3].tag;
sprintf(ZoneNumTag->fld, "%02d", config->utm.zoneNum);
sprintf(ZoneLtrTag->fld, "%c",   config->utm.zoneLtr);

OpenForm ( fm, 10, ob->area.Ymin ); 

do 
	{
	retv = ProcessObList ( fm->ob_lst, fm->ob_cnt, 0 );
	switch ( retv ) 
		{
		case OKAY:
			// get the field values in to radians
			config->utm.zoneNum = atoi(ZoneNumTag->fld);
			config->utm.zoneLtr = ZoneLtrTag->fld[0];
			break;

		case CANCEL-1:
			break;

		default : 
			break;
		}
	} while (retv != OKAY && retv != DONE && retv != NOT_SET);

CloseForm   ( fm );
DestroyFRFForm ( fm );

return retv;
}



/*----------------------------------------------------------------------------*/
// do UTM to Lat Lon
/*----------------------------------------------------------------------------*/
int doUTMtoLatLon ( ob_desc * ob )
{
int retv;
fm_desc * fm;
char * frf = "utm_toll.frf";
ftag * LatTag;
ftag * LonTag;
ftag * ZoneLtrTag;
ftag * ZoneNumTag;
ftag * utmXTag;
ftag * utmYTag;
char buf[80];
double utmX, utmY;
int grid_num;
char grid_ltr;
double scale_factor;
t_setup * config = QuerySetup();
t_ellips * ellipse = query_ellipsoid();   // ellipsoid for arclength comps

// Load the FRF file 
if ( ( fm = loadFRF ( frf ) ) == NULL ) 
	{
  return FALSE;
	}

fm->ob_lst[6].flags &= ~IS_EDITABLE;
fm->ob_lst[7].flags &= ~IS_EDITABLE;

OpenForm ( fm, 10, ob->area.Ymin ); 

// set up form
LatTag       = (ftag *) fm->ob_lst[2].tag;
LonTag       = (ftag *) fm->ob_lst[3].tag;
ZoneNumTag   = (ftag *) fm->ob_lst[4].tag;
ZoneLtrTag   = (ftag *) fm->ob_lst[5].tag;
utmXTag      = (ftag *) fm->ob_lst[6].tag;
utmYTag      = (ftag *) fm->ob_lst[7].tag;

// update position
PositionString(LonTag->fld, 
               LatTag->fld, 
               buf, 
               lastFrom.lon, 
               lastFrom.lat, 
               0.0, 0);

// update UTM Zone and Letter
grid_num = config->utm.zoneNum;
grid_ltr = config->utm.zoneLtr;
sprintf(ZoneNumTag->fld, "%02d", config->utm.zoneNum);
sprintf(ZoneLtrTag->fld, "%c",   config->utm.zoneLtr);

// clear new UTM coords
strset(utmXTag->fld, '\0');
strset(utmYTag->fld, '\0');

DrawTaggedOb(&fm->ob_lst[2]); 
DrawTaggedOb(&fm->ob_lst[3]); 
DrawTaggedOb(&fm->ob_lst[4]); 
DrawTaggedOb(&fm->ob_lst[5]); 
DrawTaggedOb(&fm->ob_lst[6]); 
DrawTaggedOb(&fm->ob_lst[7]); 

SetFldExitTask( PositionExitTask );

do 
	{
	retv = ProcessObList ( fm->ob_lst, fm->ob_cnt, 0 );
	switch ( retv ) 
		{
		case CALCULATE:
			// get the field values in to radians
			if (!PositionStrToRads(LatTag->fld, &lastFrom.lat, LAT))
				retv = 99;
			
			if (!PositionStrToRads(LonTag->fld, &lastFrom.lon, LON))
				retv = 99;

			// get the zone info
			grid_num = atoi(ZoneNumTag->fld);
			grid_ltr = ZoneNumTag->fld[0];

			ll_to_utm(lastFrom.lat, 
          lastFrom.lon,                                // current pos
          &utmY, &utmX,                                // returns UTM xy 
          &scale_factor,                               // returns scale factor
          &grid_num,                                   // returns zone number
          &grid_ltr,                                   // returns zone letter
          USE_PASSED_ZONE,                             // use the zone specified
          ellipse->a,                                  // semi maj axis
          ellipse->e2);                                // semi minor axis

			sprintf(utmYTag->fld, "%.1lf",   utmY);
			sprintf(utmXTag->fld, "%.1lf",   utmX);
			DrawTaggedOb(&fm->ob_lst[6]); 
			DrawTaggedOb(&fm->ob_lst[7]); 

			break;

		case DONE:
			break;

		case COMPUTE_UTM_ZONE:
			if (!PositionStrToRads(LatTag->fld, &lastFrom.lat, LAT))
				retv = 99;
			
			if (!PositionStrToRads(LonTag->fld, &lastFrom.lon, LON))
				retv = 99;

			sprintf(ZoneNumTag->fld, "%02d", grid_zone_EW(lastFrom.lon));
			sprintf(ZoneLtrTag->fld, "%c",   grid_zone_NS(lastFrom.lat));
			DrawTaggedOb(&fm->ob_lst[4]); 
			DrawTaggedOb(&fm->ob_lst[5]); 
			break;

		default : 
			break;
		}
	} while (retv != DONE && retv != NOT_SET);

CloseForm   ( fm );
DestroyFRFForm ( fm );

return retv;
}

