/*----------------------------------------------------------------------------*/
// File: relay.c   USGS Navigation, handle the contact closure board io
// Blame: John Gann
// Start Date: 1 June 1992
/*----------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <conio.h>
#include <time.h>

#include "navdefs.h"
#include "navtypes.h"
#include "nav_util.h"
#include "relay.h"

static unsigned int baseAddress = 0x330;              // address of PDISO 8
static unsigned char relayByte  = 0x00;               // byte register val
static char relayStatus        = SUCCESS;             // state of board
static time_t lastTime[MAX_RELAYS];                   // time of last xmission
#define WAIT_FOR_IO_BUS() inp(baseAddress+1)

/*----------------------------------------------------------------------------*/
// initialize relay IO address
/*----------------------------------------------------------------------------*/
int initRelayIO(void)
{
t_setup * config = QuerySetup();
unsigned char byte;

if (!config->relay.enabled)
	{
	return RELAY_NOT_ENABLED;
	}

baseAddress = config->relay.address;
setAllRelays();                          // Set all registers on
byte = readRelayByte();                  // Read them back
if (byte != 0xFF)                        // Were they right ??
	{
	relayStatus = RELAY_BOARD_NOT_FOUND;
	printf("relay board not found\n");
	}
else
	{
	relayStatus = SUCCESS;
	clearAllRelays();
	}
return relayStatus;
}


/*----------------------------------------------------------------------------*/
// Set all enabled relays ON
/*----------------------------------------------------------------------------*/
void setAllRelays(void)
{
t_setup * config = QuerySetup();

if (relayStatus != SUCCESS)
	return;

outp(baseAddress, relayByte = 0xFF);
}



/*----------------------------------------------------------------------------*/
// Set the indexed relay on or off
/*----------------------------------------------------------------------------*/
int setRelay(int idx, int state)
{
if (relayStatus != SUCCESS)
	return -1;

WAIT_FOR_IO_BUS();
readRelayByte();

if (state)
	{
	relayByte |= (unsigned char) (state << idx);
	}
else
	{
	relayByte &= ((unsigned char) (~(1 << idx)));
	}


// This delay is just a near jump required to allow settling time on the IO bus
// Take it out and the 2nd IO operation will fail
WAIT_FOR_IO_BUS();

return outp(baseAddress, relayByte);
}

/*----------------------------------------------------------------------------*/
// Clear all enabled relays ON
/*----------------------------------------------------------------------------*/
int clearAllRelays(void)
{
if (relayStatus != SUCCESS)
	return -1;

relayByte = 0x00;
return outp(baseAddress, relayByte);
}

/*----------------------------------------------------------------------------*/
// Read the current state of the relays
/*----------------------------------------------------------------------------*/
unsigned char readRelayByte(void)
{
if (relayStatus != SUCCESS)
	return 0;

relayByte = (unsigned char) inp(baseAddress);
return relayByte;
}


/*----------------------------------------------------------------------------*/
// take care of setting the relays
/*----------------------------------------------------------------------------*/
int doRelays(void)
{
int i;
t_setup * config = QuerySetup();
time_t now = time(NULL);
int relayCnt = 0;
time_t elapsedTime;

// Is the Relay Board Installed
if (!config->relay.enabled)
	return RELAY_NOT_ENABLED;

// first see if its time to turn any open relays off
openRelays();

// walk through the last time a relay was hit 
for (i=0; i<MAX_RELAYS; i++)
	{
	if ((config->relay.relaysEnabled[i]) &&
      (now - lastTime[i] >= (time_t) config->relay.relayIntervals[i]) &&
      ((now % (time_t) config->relay.relayIntervals[i]) == 0L) )
		{
		elapsedTime = now - config->relay.lastTime[i];
		if (elapsedTime >= (time_t) config->relay.relayIntervals[i])
			{
			setRelay(i, RELAY_ON);
			config->relay.lastTime[i] = now;
			relayCnt++;
			}
		}
	}
return relayCnt;
}




/*----------------------------------------------------------------------------*/
// Open any closed relays that have been closed longer than RELAY_CLOSE_TIME
/*----------------------------------------------------------------------------*/
int openRelays(void)
{
int i;
t_setup * config = QuerySetup();
time_t now = time(NULL);
int relayCnt = 0;
time_t closedTime;
unsigned char byte = readRelayByte();               // get status of relays

// walk through the last time a relay was hit 
for (i=0; i<MAX_RELAYS; i++)
	{
	if (config->relay.relaysEnabled[i])
		{
		if (relayByte & (1 << i))            // Is this relay currently open ??
			{
			closedTime = now - config->relay.lastTime[i];
			if (closedTime >= RELAY_CLOSE_TIME)
				{
				relayCnt++;
				setRelay(i, RELAY_OFF);
				}
			}
		}
	}
return relayCnt;
}



/*----------------------------------------------------------------------------*/
// Read input byte
/*----------------------------------------------------------------------------*/
int readIOLine(int which)
{
unsigned char ioByte;

if (relayStatus != SUCCESS)
	return -1;

ioByte = inp(baseAddress+1);

return (ioByte & (0x01 << which)) ? 1 : 0;
}

