/*----------------------------------------------------------------------------*/
// File: sites.c   USGS Navigation, allow user to manipulate sampling sites
// Author: John Gann
// Start Date: 25 May 1992
/*----------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <time.h>
#include <fcntl.h>
#include <io.h>

#include <GRconst.h>
#include <GRports.h>
#include <GRextrn.h>
#include <menuet.h>

#include "navdefs.h"
#include "navtypes.h"
#include "utmlib.h"
#include "dispasci.h"
#include "nav.h"                 // Prototypes
#include "geo_utils.h"           // Prototypes
#include "lines.h"               // Prototypes
#include "nav_util.h"           // Prototypes
#include "sensor.h"             // Prototypes
#include "mnutils.h"            // Prototypes
#include "sites.h"              // Prototypes
#include "video.h"              // Prototypes
#include "printer.h"            // Prototypes

// Head of site doubly linked list
static t_site * siteHead = (t_site *) NULL;
static t_site * lastSite = (t_site *) NULL;
static fm_desc * siteFm = (fm_desc *) NULL;
static char * CANT_OPEN_MSG = "Can't Open";
static char * WARNING_MSG   = "Yo";
// externals
extern char RefreshRequired;

/*------------------------------------------------------------------*/
// the Add Sites function
/*------------------------------------------------------------------*/
int doAddSite(ob_desc * ob)
{
int retv;

// load the site file into memory if it exists
load_sitefile();

retv = doSiteForm(ob, lastSite, NULL, ADD);

// Save the sites
write_site_file();

return retv;
}


/*------------------------------------------------------------------*/
// the Display Sites function
/*------------------------------------------------------------------*/
int doDisplaySites(ob_desc * ob)
{
// load the site file into memory if it exists and has not yet been loaded
load_sitefile();

return pickSites(ob, "Select Sites for Display", DISPLAY);
}


/*------------------------------------------------------------------*/
// the Delete Sites function
/*------------------------------------------------------------------*/
int doDeleteSites(ob_desc * ob)
{
// load the site file into memory if it exists and has not yet been loaded
load_sitefile();

return pickSites(ob, "Select Sites for Deletion", DELETE);
}


/*------------------------------------------------------------------*/
// the Print Sites function
/*------------------------------------------------------------------*/
int doListSites(ob_desc * ob)
{
char tmpname[_MAX_PATH] = {"nvXXXXXX"};
t_site * aSite = siteHead;
FILE * fp;

// Create a temp file name to store the output to.
if (mktemp(tmpname) == NULL)
	{
	return UNABLE_TO_CREATE_TMPFILE;
	}
else
	{
	if ((fp = fopen(tmpname, "w")) == (FILE *) NULL)
		{
		return BAD_FILE_OPEN;
		}
	}

// load the site file into memory if it exists and has not yet been loaded
load_sitefile();
aSite = siteHead;

while (aSite != (t_site *) NULL)
	{
	printSite(fp, aSite);
	aSite = aSite->next;
	}
	
fclose(fp);

// get a temp file name
DisplayAsciiFile( tmpname, 1, 32, 10, 50);

// get rid of temp file
remove(tmpname);

return SUCCESS;
}


/*------------------------------------------------------------------*/
// function to load the binary sitefile
/*------------------------------------------------------------------*/
int load_sitefile(void)
{
FILE * fp;
t_site aSite;
int retv;

// if the sites have already been read in, return
if (siteHead != (t_site *) NULL)
	return SITES_ALREADY_LOADED;

// attempt to open the sitefile
if ((fp = fopen(SITE_FILE_NAME, "r+b")) == (FILE *) NULL)
	{
	return BAD_FILE_OPEN;
	}

// read the file in mallocing as you go
while((retv = getNextLine(fp, &aSite, sizeof(t_site))) == SUCCESS)
	{
	// add the site to the linked list
	if ((retv = addSiteItem(&aSite)) != SUCCESS)
		{
		fclose(fp);
		printf("addSiteItem() failed with %d\n", retv);
		return retv;
		}
	}

fclose(fp);
return TRUE;
}


/*----------------------------------------------------------------------------*/
// Add an item to the end of the doubly linked sitefile list
// return the pointer to the next node if successful, else return NULL
/*----------------------------------------------------------------------------*/
int addSiteItem(t_site * newSite)
{
t_site * last = getLastSite(); 
t_site * new;

if (newSite == (t_site *) NULL)
	{
	return NULL_SITE;
	}

if (last == (t_site *) NULL)            // is the list empty
	{
	if ((last = (t_site *) malloc(sizeof(t_site))) == (t_site *) NULL)
		{
		printf("malloc failed in %s at site %d\n", __FILE__, __LINE__);
		return BAD_MALLOC;
		}
	newSite->previous = (t_site *) NULL;
	newSite->next = (t_site *) NULL;
	memcpy((void *) last, (void *) newSite, sizeof(t_site));
	siteHead = last;
	lastSite = last;
	}
else
	{
	if ((last->next = (t_site *) malloc(sizeof(t_site))) == (t_site *) NULL)
		{
		printf("malloc failed in %s at site %d\n", __FILE__, __LINE__);
		return BAD_MALLOC;
		}

	memcpy((void *) last->next, (void *) newSite, sizeof(t_site));
	new = last->next;
	new->previous = last;
	new->next = (t_site *) NULL;

	lastSite = last->next;
	}

return SUCCESS;
}



/*----------------------------------------------------------------------------*/
// Retun the number of sites in the list
/*----------------------------------------------------------------------------*/
int getSiteCount(void)
{
t_site * aSite = siteHead;
int i=0;

load_sitefile();
while (aSite != (t_site *) NULL)
	{
	aSite = aSite->next;
	i++;
	}

return i;
}



/*----------------------------------------------------------------------------*/
// Return the Site in the linked list or that matches the name
/*----------------------------------------------------------------------------*/
t_site * getNamedSite(char * name)
{
t_site * aSite = siteHead;

while (aSite != (t_site *) NULL)
	{
	if ((strcmp(aSite->name, name)) == SUCCESS)
		return aSite;

	aSite = aSite->next;
	}

return (t_site *) NULL;
}


/*----------------------------------------------------------------------------*/
// Retun the Nth Site in the linked list or Null if not found
/*----------------------------------------------------------------------------*/
t_site * getNthSite(int n)
{
t_site * aSite = siteHead;
int i=0;

// scrollMsg("getNthSite()\n");
while (aSite != (t_site *) NULL)
	{
	if (i == n)
		return aSite;

	aSite = aSite->next;
	i++;
	}

return (t_site *) NULL;
}




/*----------------------------------------------------------------------------*/
// print a site
/*----------------------------------------------------------------------------*/
void printSite(FILE * fp, t_site * aSite)
{
char x_buf[40];
char y_buf[40];
char z_buf[40];

// sanity checks
if (aSite == (t_site *) NULL)
	return;

// sanity checks
if (fp == (FILE *) NULL)
	return;

PositionString(x_buf, y_buf, z_buf, aSite->pos.lon, aSite->pos.lat, 0.0, 0);
fprintf(fp, "    Name: %s\nPosition: %-18s %-18s\n", aSite->name, y_buf, x_buf);
fprintf(fp, "   Depth: %.1lf\n", aSite->depth);
fprintf(fp, "    Type: %s\n\n", querySampleTypeName(aSite->type));
}



/*----------------------------------------------------------------------------*/
// Delete the Nth site from the linked list
/*----------------------------------------------------------------------------*/
int deleteSiteList(char marks[])
{
t_site * aSite = siteHead;
int i=0;
int max_sites = getSiteCount();
t_site * next;

for (i=0; i<max_sites; i++)
	{
	next = aSite->next;
	if (marks[i] == (char) '+')
		{
		deleteSite(aSite);
		}

	aSite = next;
	}

write_site_file();
return TRUE;
}


/*----------------------------------------------------------------------------*/
// Delete the passed t_site *
/*----------------------------------------------------------------------------*/
int deleteSite(t_site * aSite)
{
t_site * previous = aSite->previous;
t_site * next = aSite->next;

if (siteHead == (t_site *) NULL)
	{
	return FALSE;
	}

if (aSite == (t_site *) NULL)
	{
	return FALSE;
	}

if (aSite == siteHead)
	{
	if (siteHead->next == (t_site *) NULL)
		// there are no more sites after this one
		{
		free(aSite);
		siteHead = (t_site *) NULL;
		lastSite = (t_site *) NULL;
		aSite = (t_site *) NULL;
		return TRUE;
		}
	siteHead = next;
	siteHead->previous = (t_site *) NULL;
	free(aSite);
	return TRUE;
	}

if (aSite == lastSite)
	{
	if (lastSite->previous == (t_site *) NULL)
		{
		// there are no more sites after this one
		free(aSite);
		siteHead = (t_site *) NULL;
		lastSite = (t_site *) NULL;
		aSite = (t_site *) NULL;
		return TRUE;
		}
	free(aSite);
	lastSite = previous;
	lastSite->next = (t_site *) NULL;
	return TRUE;
	}

// This site is neither the first or the last
aSite->previous->next = next;
aSite->next->previous = previous;
free(aSite);

return TRUE;
}


/*----------------------------------------------------------------------------*/
// return the last item in the doubly linked site list
// if there are no sites return NULL                -- empty list
// if there is one site return siteHead             -- one item
// if there is more than site return lastSite       -- several items
/*----------------------------------------------------------------------------*/
t_site * getLastSite( void )
{
if (lastSite == (t_site *) NULL)
	{
	return siteHead;            // either one or none sites in linked list
	}

return lastSite;              // must be greater han one site in list
}


/*---------------------------------------------------------------------------*/
// The Site Form
/*---------------------------------------------------------------------------*/
int doSiteForm(ob_desc * ob, t_site * aSite, char * title, char mode)
{
static int idx=0;
int retv;
int retval;
char buf[256];
char x_buf_1[30];         /* String for Starting Lon or Easting */
char y_buf_1[30];         /* String for Starting Lat or Northing */
int CoordinateSystem =  QuerySetup()->CoordSys;     
int HorizUnits =        QuerySetup()->HorizUnits;  
int AngleMode =         QuerySetup()->AngleType;  
int DecimalAccuracy =   QuerySetup()->DecimalAccuracy;
t_ellips * elps = query_ellipsoid();
int zone_num;
char zone_ltr;
ftag * tag;
ltag * lTag;
t_site newSite;
t_setup * config = QuerySetup();

if ( ( siteFm = loadFRF ( "sites.frf" ) ) == (fm_desc *) NULL ) 
	return FALSE;


if (title == (char *) NULL)
	{
	strcpy(siteFm->title, "Site Entry Form");
	}
else
	{
	strcpy(siteFm->title, title);
	}

// if passed a NULL site, then just use a default site
if (aSite == (t_site *) NULL)
	{
	aSite = (t_site *) &newSite;
	}
else
	{
	memcpy((void *) &newSite, (void *) aSite, sizeof(t_site));
	}

zone_num = grid_zone_EW(aSite->pos.lon);
zone_ltr = grid_zone_NS(aSite->pos.lat);

if (!OpenForm( siteFm, 10, ob->area.Ymax))
	{
	sprintf(buf, "Failed to open form in %s at site %d\n", __FILE__, __LINE__);
	Puke(buf);
	}

// Initialize the form with the passed site
UpdatePositionEntryFields(siteFm, 1, aSite->pos.lon, aSite->pos.lat); 

tag = (ftag *) siteFm->ob_lst[5].tag;
strcpy(tag->fld, aSite->name);
DrawTaggedOb(&siteFm->ob_lst[5]); 

siteFm->ob_lst[8].flags  |= IS_EDITABLE;
tag = (ftag *) siteFm->ob_lst[8].tag;
sprintf(tag->fld, "%.1lf", aSite->depth);
DrawTaggedOb(&siteFm->ob_lst[8]); 

lTag = (ltag *) siteFm->ob_lst[10].tag;
lTag->curitem = aSite->type;
DrawTaggedOb(&siteFm->ob_lst[10]); 

SetFldExitTask( PositionExitTask );


do 
	{
	retv = ProcessObList ( siteFm->ob_lst, siteFm->ob_cnt, 0 );
	switch ( retv ) 
		{
		case OKAY : 
			/* See if all of the fields are valid */
			tag = (ftag *) siteFm->ob_lst[6].tag;
			strcpy(y_buf_1, tag->fld);
			tag = (ftag *) siteFm->ob_lst[7].tag;
			strcpy(x_buf_1, tag->fld);
			if (PositionXYToRads(y_buf_1, x_buf_1, 
                           &newSite.pos.lat, &newSite.pos.lon, 
                           zone_num, zone_ltr))
				{
				retv = 99;        /* Stay in LOOP */
				break;
				}

			// Get the site depth
			tag = (ftag *) siteFm->ob_lst[8].tag;
			newSite.depth = atof(tag->fld);

			// Get the site name
			tag = (ftag *) siteFm->ob_lst[5].tag;
			strcpy(newSite.name, tag->fld);

			// Get the display flag
			if (siteFm->ob_lst[9].flags & IS_CHECKED)
				{
				newSite.display = TRUE;
				}
			else
				{
				newSite.display = FALSE;
				}

			// Get the site type
			newSite.type = lTag->curitem;

			switch (mode)
				{
				case ADD:
					// add the Site to the Linked list
					if ((retval = addSiteItem(&newSite)) != SUCCESS)
						{
						printf("addSiteItem() failed with %d\n", retval);
						}
					break;

				case EDIT:
					// if they are editing current site update that too
					if (!memcmp((void *) &config->curSite.name, 
                      (void *) &aSite->name, sizeof(aSite->name)))
						{
						memcpy((void *) &config->curSite, 
                 (void *) &newSite, sizeof(t_site));
						NotifyUser(-1, -1, newSite.name, "Edited currently selected site");
						}

					// change the Site in the Linked list
					memcpy((void *) aSite, (void *) &newSite, sizeof(t_site));
					break;
				}

			break;

		case DEFAULT : 
			UpdatePositionEntryFields(siteFm, 1, 
                                    aSite->pos.lon, aSite->pos.lat); 
			
			// update the site name
			tag = (ftag *) siteFm->ob_lst[5].tag;
			strcpy(tag->fld, aSite->name);
			DrawTaggedOb(&siteFm->ob_lst[5]); 
			break;

		case CANCEL : 
			break;

		case INFO : 
			if (aSite->historyFile[0] == '\0')
				{
				DisplayAsciiFile( aSite->historyFile, 1, 32, 10, 50);
				}
			else
				{
				NotifyUser(-1, -1, WARNING_MSG, "No History Yet");
				}
			break;

		case VIDEO : 
			// NotifyUser(-1, -1, WARNING_MSG, dumpVideoHook(&aSite->vHook, buf));
			if (aSite->vHook.fileName[0] != '\0')
				{
				retval = displayVideoFile( &aSite->vHook);
				if (retval != SUCCESS)
					{
					sprintf(buf, "displayVideoFile Failed with %d at %s:%d\n", 
                        retval, 
                        __FILE__, 
                        __LINE__);
					NotifyUser(-1, -1, WARNING_MSG, buf);
					}
				}
			else
				{
				NotifyUser(-1, -1, WARNING_MSG, "No Video Recorded");
				}
			break;
		}
	} while (retv != OKAY && retv != CANCEL);


CloseForm ( siteFm );
DestroyFRFForm(siteFm);
RefreshRequired = (char) (retv == OKAY);
return  (retv == OKAY);
}




/*----------------------------------------------------------------------------*/
// Save the site file 
/*----------------------------------------------------------------------------*/
int write_site_file(void)
{
FILE *fp;
int  k;
char buf[80];
t_site * aSite = siteHead;

if ((fp = fopen(SITE_FILE_NAME, "wb")) == NULL)
  {
  printf("\nCan't open file %s", SITE_FILE_NAME);
  return FALSE;
  }

while (aSite != (t_site *) NULL)
	{
	k = fwrite( (void *) aSite, sizeof(t_site), 1, fp);
	if (k == -1)
		{
		sprintf(buf, "Write failed %d in %s at site %d\n", k, __FILE__, __LINE__);
		Puke(buf);
		}
	aSite = aSite->next;
	}

fclose(fp);
return TRUE;
}





/*------------------------------------------------------------------*/
// the Select Site for nav function
/*------------------------------------------------------------------*/
int doSelectSite(ob_desc * ob)
{
// load the site file into memory if it exists and has not yet been loaded
load_sitefile();

return pickSite(ob, "Select Site For Steering", SELECT);
}



/*------------------------------------------------------------------*/
// the Edit Sites function
/*------------------------------------------------------------------*/
int doEditSite(ob_desc * ob)
{
int retv;

// load the site file into memory if it exists and has not yet been loaded
load_sitefile();

retv = pickSite(ob, "Pick a Site to Edit", EDIT);

// Save the sites
write_site_file();

return retv;
}


/*------------------------------------------------------------------*/
// Pick a single Site from a list box
/*------------------------------------------------------------------*/
int pickSite(ob_desc * ob, char * title, char mode)
{
fm_desc *lf;
int max_length = 0, i, j;
char *names [ MAX_SITES ];
int cur_slot, top_slot, retv, count = 0;
t_site * aSite;
static int last_slot = 0;                   // last recently used slot
char buf[512];
t_setup * config = QuerySetup();

/* load the site names */
if ((count = loadSiteNames(names, &max_length)) <= 0)
   {
   NotifyUser(-1, -1, WARNING_MSG, "No Sites in the File." );
   return -1;
   }

/* create a list form */
lf = CreateListForm ( title, 7, 4, max_length+2, 8, 0 );

lf->ob_lst [  9 ].label  = "Okay";
lf->ob_lst [  9 ].flags |= IS_EXITABLE;
lf->ob_lst [  9 ].key    = 'O';
lf->ob_lst [  9 ].state  = 8;
   
lf->ob_lst [ 10 ].label  = "Default";
lf->ob_lst [ 10 ].flags |= IS_EXITABLE;
lf->ob_lst [ 10 ].key    = 'D';
lf->ob_lst [ 10 ].state  = 8;
   
lf->ob_lst [ 11 ].label  = "Cancel";
lf->ob_lst [ 11 ].flags |= IS_EXITABLE;
lf->ob_lst [ 11 ].key    = 'C';
lf->ob_lst [ 11 ].state  = 8;
   
lf->ob_lst [ 12 ].label  = "Help";
lf->ob_lst [ 12 ].flags |= IS_EXITABLE;
lf->ob_lst [ 12 ].key    = 'H';
lf->ob_lst [ 12 ].state  = 8;

cur_slot = top_slot = (last_slot < count && last_slot >= 0) ? last_slot : 0 ;

/* open a form to put the list on */
OpenForm( lf, 10, ob->area.Ymin);

do {
retv = ProcessListForm(lf, names, count, &top_slot, &cur_slot );

switch ( retv ) 
	{
	case 9 :        // Okay
		last_slot = cur_slot;
		aSite = getNthSite(cur_slot);
		if (aSite == (t_site *) NULL)
			{
			NotifyUser(-1, -1, "Error", "Unable to match site\n");
			}
		else
			{
			switch (mode)
				{
				case EDIT:
					sprintf(buf, "Edit Site %d of %d\n", cur_slot+1, getSiteCount());
					if (doSiteForm(ob, aSite, buf, EDIT))
						{
						loadSiteNames(names, &max_length);
						if ((count = loadSiteNames(names, &max_length)) <= 0)
   						{
   						NotifyUser(-1, -1, WARNING_MSG, "No Sites in the File." );
   						}
						else
							{
							DrawForm(lf);
							}
						}
					break;

				case SELECT:
					// Store the current site in the configuration file
					memcpy((void *) &config->curSite, 
                 (void *) aSite, sizeof(t_site));
					querySiteInfo()->initialized = FALSE;
					j = sprintf(buf, "Selected New Site\n");
					dumpSite(&config->curSite, buf+j);
					printMsg(buf);
					logMsg(buf);
					break;
				}   // end of switch

			RefreshRequired = TRUE;
	// 		resetTrack(aSite);
			}
		break;
   
	case 10:        // Default
		break;

	case 11 :       // Cancel
		break;
   
	case 12 :       // Help
		break; 
	}
} while ( (retv != NOT_SET) && ( retv != 11 ) && (retv != 9) );

// clean up alloced memory
i = count;
while ( i ) 
	free ( names [ --i ] );

CloseForm   ( lf );
DestroyForm ( lf );


if (retv == 9)
	{
	return cur_slot;
	}
else
	{
	return -1;
	}
}




/*------------------------------------------------------------------*/
// Pick several sites from a checklist
/*------------------------------------------------------------------*/
int pickSites(ob_desc * ob, char * title, char mode)
{
fm_desc *lf;
int max_length = 0, i;
char *names [ MAX_SITES ];
int cur_slot, top_slot, retv, count = 0;
char marks[MAX_SITES];

memset((void *) marks, ' ', sizeof(marks));

if ((count = loadSiteNames(names, &max_length)) <= 0)
   {
   NotifyUser(-1, -1, WARNING_MSG, "No Sites in the File." );
   return -1;
   }

if (mode == DISPLAY)
	{
	siteDisplayFlags(marks, GET);
	}

/* create a list form */
lf = CreateChkListForm( title, 7, 4, max_length+2, 7);
lf->ob_lst[9].label = "Okay";
lf->ob_lst[9].flags |= IS_EXITABLE;
lf->ob_lst[9].key = 'O';
lf->ob_lst[9].state = 8;

lf->ob_lst[10].label = "All";
lf->ob_lst[10].flags |= IS_EXITABLE;
lf->ob_lst[10].key = 'A';
lf->ob_lst[10].state = 8;

lf->ob_lst[11].label = "None";
lf->ob_lst[11].flags |= IS_EXITABLE;
lf->ob_lst[11].key = 'N';
lf->ob_lst[11].state = 8;

lf->ob_lst[12].label = "Cancel";
lf->ob_lst[12].flags |= IS_EXITABLE;
lf->ob_lst[12].key = 'C';
lf->ob_lst[12].state = 8;

cur_slot = top_slot = 0;

/* open a form to put the list on */
OpenForm( lf, ob->area.Xmin-40, ob->area.Ymax);

do 
	{
	retv = ProcessChkListForm( lf, names, marks, count, &top_slot);
	switch ( retv ) 
		{
		case 9 :        // Okay
			switch (mode)
				{
				case DISPLAY:
					RefreshRequired = TRUE;
					siteDisplayFlags(marks, SET);
					break;

				case DELETE:
					RefreshRequired = TRUE;
					deleteSiteList(marks);
					break;
				}
			break;
   
		case 10:        // All On
			for (i=0; i< count; i++)
				marks[i] = '+';
			top_slot = 0;
			DrawForm(lf);
			break;
	
		case 11 :       // All Off
			for (i=0; i< count; i++)
				marks[i] = ' ';
			top_slot = 0;
			DrawForm(lf);
			break;
  	 
		case 12 :       // Cancel
			break; 
		}
	} while ( (retv != 9) && (retv != NOT_SET) && ( retv != 12 ) );

// clean up alloced memory
i = count;
while ( i ) 
	free ( names [ --i ] );

CloseForm   ( lf );
DestroyForm ( lf );

if (retv == 9)
	{
	return cur_slot;
	}
else
	{
	return -1;
	}
}


/*----------------------------------------------------------------------------*/
// Load the site names from the linked list
/*----------------------------------------------------------------------------*/
int loadSiteNames(char * names[], int * max_length)
{
int i = 0;
int count = 0;
t_site * aSite;

while(count < MAX_SITES && (aSite = getNthSite(count)))
   {
   names [ count ] = ( char * ) malloc ( strlen ( aSite->name )+1 );

   if ( names [ count ] == (char *) NULL ) 
      {
      i = count;
      while ( i ) 
				free ( names [ --i ] );
      NotifyUser(-1, -1, "ERROR", "Not Enough Memory." );
      return -1;
      }
   else 
      {
      strcpy ( names [ count++ ], aSite->name );
      *max_length = max ( *max_length, (int) strlen ( aSite->name ) );
      }
	}  /* end of while */
return count;
}



/*----------------------------------------------------------------------------*/
// display the passed site
/*----------------------------------------------------------------------------*/
char * dumpSite(t_site * aSite, char * buf)
{
int j = 0;
char latBuf[40];
char lonBuf[40];
char altBuf[40];
char vBuf[256];

if (aSite == (t_site *) NULL)
	{
	j += sprintf(buf+j, "Null Site");
	}
else
	{
	PositionString(lonBuf, latBuf, altBuf, 
                 aSite->pos.lon, 
                 aSite->pos.lat, 0.0, 0);
	j += sprintf(buf+j, "   Name : %s\n", aSite->name);
	j += sprintf(buf+j, "   Site : %s %s\n", latBuf, lonBuf);
	j += sprintf(buf+j, "Display : %d\n", aSite->display);
	j += sprintf(buf+j, "   Type : %d\n", aSite->type);
	j += sprintf(buf+j, "%s", dumpVideoHook(&aSite->vHook, vBuf));
	}

return buf;
}




/*----------------------------------------------------------------------------*/
// Load the display flag from the linked list
/*----------------------------------------------------------------------------*/
int siteDisplayFlags(char marks[], int action)
{
int count = 0;
t_site * aSite;

while(count < MAX_SITES && (aSite = getNthSite(count)))
	{
	if (aSite == (t_site *) NULL)
		return count;

	if (action == GET)
		{
		marks [ count++ ] = (char) ((aSite->display) ? '+' : ' ');
		}
	else
		{
		aSite->display = (marks[count++] == '+') ? TRUE: FALSE;
		}

	}

return count;
}



/*----------------------------------------------------------------------------*/
// return a pointer to sample type names
/*----------------------------------------------------------------------------*/
char * querySampleTypeName(int type)
{
static char * sampleTypeNames[] = 
	{
	"Box Core\0\0\0\0",
	"Gravity Core",
	"VibraCore\0\0\0",
	"Piston Core\0",
	"Dredge\0\0\0\0\0\0",
	"Camera Run\0\0",
	"Water Sample",
	"Ice Sample\0\0",
	"Dart Core\0\0\0",
	"Misc Sample\0"
	};


if (type >= 0 && type < sizeof(sampleTypeNames) / sizeof(sampleTypeNames[0]) )
	{
	return sampleTypeNames[type];
	}

return "Unknown";
}


/*----------------------------------------------------------------------------*/
// find the site that matches this one
/*----------------------------------------------------------------------------*/
t_site * queryMatchingSite(t_site * thisSite)
{
t_site * aSite;

// load the site file into memory if it exists
load_sitefile();

aSite = siteHead;
while (aSite != (t_site *) NULL)
	{
	if (!memcmp((void *) thisSite->name, 
              (void *) aSite->name, 
              sizeof(aSite->name)))
		{
		if (thisSite->type == aSite->type)
			{
			// Must be the one
			return aSite;
			}
		}
	aSite = aSite->next;
	}

return (t_site *) NULL;
}
