/* !sccs!     File: utmlib.c    Version: 1.7  8/20/91 10:20:15  */
/******************************************************************************/
/******************************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include "navdefs.h"

#define COMPUTE_NEW_ZONE 1
#define USE_PASSED_ZONE 0
#define ILLEGAL_SQRT      -1
#define BAD_LAT           -2
#define BAD_LON           -3
#define FUDGE .00000004  /* (rads) approx .1 meters */  
#define sign(value) ((value < 0.0) ? (-1.0) : 1.0)
int compute_cm_from_zone(int zone);

/*----------------------------------------------------------------------------*/
/* return the number of the UTM grid zone 																		*/
/*----------------------------------------------------------------------------*/
int	grid_zone_EW(double lon_rad)

{
int	izn;
double ulong = lon_rad * RAD_TO_DEG;

	/* put input angle between +/- 180 degrees */
for(;ulong < -180.0; ulong += 360.0);
for(;ulong > 180.0; ulong -= 360.0);

	/* return integer representing UTM grid zone */

izn = ( (int) (ulong / 6.0) ) + 31 + ((ulong < 0.0) ? -1 : 0);

for(;izn < 1; izn+=60);
for(;izn > 60; izn -=60);

return(izn);

}	/* end of  grid_zone_EW */





/*----------------------------------------------------------------------------*/
/* Compute and return the grid zone character.  Input is user latitude in     */
/* degrees and should be between -80 deg ( 80 deg south ) and +84 deg         */
/* ( 84 deg north).  If less than  -80 deg ,  'A'  is returned; If greater    */
/* than +84 deg, 'Z'  is returned. 																						*/
/*----------------------------------------------------------------------------*/
char	grid_zone_NS(double lat_rad)
{
static	char	zc[22] = {'A', 'C', 'D', 'E', 'F', 'G', 'H', 'J', 'K', 'L', 'M',
		'N', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Z'};
double ulat = RAD_TO_DEG * lat_rad;

int	izn;

	/* put input angle between +/- 180 degrees.  Note, however,
		that latitudes should fall between +/- 90 degrees. */
for(;ulat < -180.0; ulat += 360.0);
for(;ulat > 180.0; ulat -= 360.0);

if ( ulat < -80.0 ) return( zc[0] );
if ( ulat >  84.0 ) return( zc[21] );

	/* return integer representing UTM grid zone */

izn = (int) ((ulat + 80.0) / 8.0 + 1.0);
if (ulat < -80.0) izn = 0;
if (ulat >= 72.0) izn = 20;
if (ulat >= 84.0) izn = 21;

return( zc[izn] );

}	/* end of  grid_zone_NS */




/*----------------------------------------------------------------------------*/
/* Returns central meridian in degrees.  If latitude is out of limits, then   */
/* the letter will be 'A'  for further south than 80 deg S or  'Z'  for       */
/* further north than 84 deg N 																								*/
/*----------------------------------------------------------------------------*/
int	cent_med(double lat_rad, double lon_rad, 
                            int * gz_num, char *gz_ltr, 
                            int mode)
{
double ulat = lat_rad * RAD_TO_DEG;
double ulong = lon_rad * RAD_TO_DEG;

	/* Get normal grid zone data */
if (mode == COMPUTE_NEW_ZONE)
	{
	*gz_ltr = grid_zone_NS(lat_rad);
	*gz_num = grid_zone_EW(lon_rad);
	}


	/* Check for the non-standard grid zones */

	/* Non-standard grid zones are:

		31V, 0 to 3 deg long, cm is 3 deg
		32V, 3 to 12 deg long, cm is 9 deg
		31X, 0 to 9 deg long, cm is 3 deg
		33X, 9 to 21 deg long, cm is 15 deg
		35X, 21 to 33 deg long, cm is 27 deg
		37X, 33 to 42 deg long, cm is 39 deg

		Row  V  is 56 to 64 deg North; Row X is 72 to 84 deg North .

		Note that, if the grid zone number is correctly given, then
			the central meridian is correctly computed from
			the formula     6 * gzn - 183 .  This means that
			the utm to geodetic procedures do not have to
			consider the non-standard grid zone test.
			However, if an error check is to be made on the 
			input, then checking for a greater than
			three degree offset from the central meridian
			is incomplete.
	*/

if ( *gz_num == 31 && *gz_ltr == 'V' && ulong > 3.0 ) {
	*gz_num = 32;
}

if ( *gz_ltr == 'X' && ( *gz_num == 32 || *gz_num == 34 || *gz_num == 36) ) {
	*gz_num += ( ulong < 6.0 * *gz_num - 183 ? -1 : 1);
}

return( 6 * *gz_num - 183);

}	/* end of  cent_med */




/*----------------------------------------------------------------------------*/
/* Input is  ulat  and  ulong  in radians on the reference ellipsoid with     */
/* the parameters fiven in  a  and  e2 .  These are the exact equations.      */
/* Reference is USGS Professional Paper 1395, Chapter 8.  Output is northing, */
/* easting, grid zone number and letter.  If the latitude is further south    */
/* than 80 deg south or further north than 84 deg north, the function returns */
/* 0 .  The values in the  northing , easting ,  gz_num  and  gz_ltr  are     */
/* undefined.  If successful, the function returns  1 .                       */
/*----------------------------------------------------------------------------*/
int ll_to_utm(double ulat, double ulong, 
                double * northing, double * easting, 
								double * scale, int * gz_num, char * gz_ltr, int mode,
								double a, double e2)
{
double	sphi, cphi, tphi, ep2, rpv, A, t, C, M, cm, m0 = 0.0, k0 = 0.9996,
	x, y, k, temp;

if ( ulat < -80.0000001 / RAD_TO_DEG || ulat >  84.0000001 / RAD_TO_DEG ) return( FALSE );


sphi = sin(ulat);
tphi = tan(ulat);
cphi = cos(ulat);

	/* minor eccentricity squared (Eqn 8-12) */
ep2 = e2 / ( 1.0 - e2);		

	/* radius of curvature in the prime vertical (Eqn 4-20) */
rpv = a / sqrt( 1.- e2 * sphi*sphi); 
t = tphi * tphi;
C = ep2 * cphi * cphi;		/* Eqn 8-14 */

cm = cent_med(ulat, ulong, gz_num, gz_ltr, mode) / RAD_TO_DEG;

A = cphi * (ulong - cm);	/* Eqn 8-15 */

	/* Eqn 3-21 */
temp = 1.0 - e2 / 4.0 * (1.0 + e2 / 16.0 * ( 3.0 + 5.0 * e2 / 4.0) );
M = temp * ulat;
temp = 3.0 * e2 / 8.0 * ( 1.0 + e2 / 4.0 * ( 1.0 + 15.0 * e2 / 32.0) );
M -= temp * sin(ulat+ulat);
temp = 15.0 * e2 * e2 / 256.0 * ( 1.0 + 3.0 * e2  / 4.0 );
M += temp * sin( 4.0 * ulat);
temp = 35.0 * e2 * e2 * e2 / 3072.0;
M -= temp * sin( 6.0 * ulat);
M *= a;

	/* Eqns 8-9, 8-10 and 8-11 */
x = k0 * rpv * ( A + 
		( 1.0 - t + C) * A*A*A/6.0
		+
		(5.0 - 18.0 * t + t*t + 72.0 * C - 58.0 * ep2)
			* A*A*A*A*A/120.0
		);
y = k0 * ( M - m0 + rpv * tphi * (A*A/2.0 + ( 5.0 - t + 9.0 * C + 4.0 * C * C)
	* A*A*A*A / 24.0 + ( 61.0 - 58.0 * t + t*t + 600.0 * C - 330*ep2)
		* A*A*A*A*A*A / 720.0 ));
k = k0 * ( 1.0 + (1.0 + C) * A*A/2.0 
	+ ( 5.0 - 4.0 * t + 42.0 * C + 13.0 *C*C - 28.0 * ep2) * A*A*A*A/24.0
	+ (61.0 - 148.0 * t + 16.0 * t*t) * A*A*A*A*A*A / 720.0);

*northing = y;
if(ulat < 0.0) *northing = 10000000.0 + *northing;
*easting = x + 500000.0;
*scale = k;

return (1);

}	/* end of  ll_to_utm */




/*--------------------------------------------------------------------*/
/*	Compute the integral power of a double number 										*/
/*--------------------------------------------------------------------*/
double pwr(double number, int power)	/* raise number integral power */
{
double temp;
temp = 1.0;
	while (power-- > 0)
		temp *= number;
return(temp);
}







/*--------------------------------------------------------------------*/
/*	compute Sodano 1980 arclength from lat1/lon1 to lat2/lon2         */
/*--------------------------------------------------------------------*/
int arclen(double lat1, double lon1, double lat2, double lon2, 
					double *distance, double *forw_azm, double *back_azm, 
					double a, double b)

{
double sma2;			/* semi major axis squared */
double smb2;			/* semi major axis squared */
double e2;			/* second eccentricity squared */

double dellon1, dellon2;	
double temp1, temp2, temp3;
double tanpsi1, tanpsi2, psi1, psi2, az12, az21, theta1, theta2;
double betasub0, bbar2, mbar, rsub0, capa, capb, capc;
double s1, s2;

/* Check for the same position, return 0 */
if ( ( lat1 == lat2 ) && ( lon1 == lon2 ) ) {
   *distance = 0.0;
   *forw_azm = 0.0;
   *back_azm = 0.0;
   return(TRUE);
}

if (lon1 == lon2) lon1 += FUDGE;
if (lat1 == lat2) lat1 += FUDGE;

/*	check coordinates for valid values	*/
if (lat1 <= -HALFPI || lat1 >= HALFPI)
	return(BAD_LAT);

if (lat2 <= -HALFPI || lat2 >= HALFPI)
	return(BAD_LAT);

if (lon1 <= -PI || lon1 >= PI)
	return(BAD_LON);

if (lon2 <= -PI || lon2 >= PI)
	return(BAD_LON);

/* initialize semi-major and semi-minor axes	*/
sma2 = pwr(a,2);
smb2 = pwr(b,2);

e2 = (sma2 - smb2) / smb2;		/* first eccentricity */

dellon1 = lon2 - lon1;			/* delta lat / lon */
dellon2 = lon1 - lon2;

temp1 = smb2 / sma2;
temp2 = temp1 * tan( lat1 );

tanpsi1 = (smb2 / sma2) * (tan(lat1));
tanpsi2 = (smb2 / sma2) * (tan(lat2));

/* see Sodano */
psi1 = atan(tanpsi1);
psi2 = atan(tanpsi2);

az12=atan2( sin(dellon1),(tanpsi2-tanpsi1*cos(dellon1))*cos(psi1));
az21=atan2( sin(dellon2),(tanpsi1-tanpsi2*cos(dellon2))*cos(psi2));

theta1 = atan2( cos(az12), tanpsi1);
theta2 = atan2( cos(az21), tanpsi2);

temp1 = pwr(tanpsi1,2) + pwr(tanpsi2,2);
temp2 = 2.0 * tanpsi1 * tanpsi2 * cos(dellon1);
temp3 = fabs( sin (dellon1));

/* Check for illegal aqrt value */
if ( temp1-temp2 < 0.0 ) 
	{
	*distance = 0.0;
	*forw_azm = 0.0;
	*back_azm = 0.0;
	return(ILLEGAL_SQRT);
	}

betasub0 = atan(((a/b) * sqrt(temp1 - temp2)) / temp3);

temp1 = cos(betasub0);
bbar2 = smb2 * (1 + e2 * sign(temp1) * pwr(temp1,2));

mbar = (sma2 - bbar2) / (sma2 + bbar2);

temp1 = (sma2 + bbar2) / 2.0;
temp2 = 1.0 - (pwr(mbar, 2) / 16.0);

/* Check for illegal aqrt value */
if ( temp1 < 0.0 ) 
	{
	*distance = 0.0;
	*forw_azm = 0.0;
	*back_azm = 0.0;
	return(ILLEGAL_SQRT);
	}

temp3 = sqrt(temp1);

rsub0 = temp3 * temp2;

capa = (mbar / 4.0) + (0.1640625 * pwr(mbar, 3));
capb = -1.0 * (pwr(mbar,2) / 64.0);
capc = -0.0390625 * pwr(mbar,3);

s1 = rsub0 * (theta1 - (capa * sin(2.0 * theta1)) +
	(capb * sin(4.0 * theta1)) - ( capc * sin(6.0 * theta1)));

s2 = rsub0 * (theta2 - (capa * sin(2.0 * theta2)) +
	(capb * sin(4.0 * theta2)) - ( capc * sin(6.0 * theta2)));


*distance = fabs(s1 + s2);

if (az12 < 0.0)
	az12 += TWOPI;
*forw_azm = az12;

*back_azm = az21;

return(SUCCESS);
}







/*--------------------------------------------------------------------*/
/* Return the double distance from the passed point to the limits of	*/
/* the passed UTM zone.
/*--------------------------------------------------------------------*/
double dist_from_zone(double latrad, double lonrad, int ref_zone_num, 
				char ref_zone_ltr, double a, double b)
{
int actual_zone;
double central_meridian;
double west_lon, east_lon;
double dist1, dist2, brg;
int ref_num = ref_zone_num;
char ref_ltr = ref_zone_ltr;

/* what zone is this point in */
actual_zone = grid_zone_EW(lonrad);

/* We're already in the right zone */
if (actual_zone == ref_zone_num)
	return 0.0;

/* what is the central meridian of the refernce zone */
central_meridian = (double) cent_med(latrad, lonrad, &ref_num, 
																&ref_ltr, USE_PASSED_ZONE);

/* if actual zone is West of ref_zone then compute distance from the    */
/* point to the Western boundary of the ref_zone else compute distance  */
/* to the eastern boundary of the zone.  Note we are ignoring the North */
/* South zone boundaries completely.                                    */

west_lon = (central_meridian - 3.0) * DEG_TO_RAD;
east_lon = (central_meridian + 3.0) * DEG_TO_RAD;
	
arclen(latrad, lonrad, latrad, west_lon, &dist1, &brg, &brg, a, b);

arclen(latrad, lonrad, latrad, east_lon, &dist2, &brg, &brg, a, b);
 
return min(dist1, dist2);
}


/*----------------------------------------------------------------------------*/
/* return the pointer to string of UTM Zone name                              */
/*----------------------------------------------------------------------------*/
char * utm_zone_str(double lat_rad, double lon_rad)
{
static char *str = "99A";
sprintf(str, "%02d%c", grid_zone_EW(lon_rad), grid_zone_NS(lat_rad));
return str;
}



/*----------------------------------------------------------------------------*/
/* Convert UTM coords to lat lon in radians                                   */
/* Convert Transverse Mercator N & E in units same as 'a' to lat lon in rads  */
/* Taken from the State Plane Transverse Mercator routines and made just a    */
/* special case of those routines.                                            */
/*----------------------------------------------------------------------------*/
int utm_to_ll(double N, double E, double * lat, double * lon, 
								int gz_num, char gz_ltr,
                double a, double e2)
{
double e_primed_2 = e2 / (1.0 - e2);
double phi_1;
double cos_phi_1;
double sin_phi_1;
double tan_phi_1;
double sin2_phi_1;
double cos2_phi_1;
double temp, t1, t2;
double e1;
double e4 = e2 * e2;
double e6 = e2 * e4;
double e1_2;
double e1_3;
double e1_4;
double D, D2, D3, D4, D5, D6, C1, C1_2, T1, T1_2, N1, R1;
double mu;
double M;
double lon_origin;
double false_e = 500000.0;
double false_n = 0.0;
double scale_factor = 0.9996;    /* Always this for UTM */

/* is this the southern hemisphere */
if (gz_ltr <= 'M')
	{
	false_n = 10000000.0;
	}

lon_origin = compute_cm_from_zone(gz_num) * DEG_TO_RAD;

N -= false_n;
E -= false_e;

/* Eqn 8-20 */
/* M = Mo + y/ko  */
M = N / scale_factor;          /* Mo is 0.0 for UTM as lat is Equator */

/* Eqn 3-24 */
e1 = (1.0 - sqrt(1.0 - e2)) / (1.0 + sqrt(1.0 - e2));
e1_2 = e1 * e1;
e1_3 = e1_2 * e1;
e1_4 = e1_2 * e1_2;

/* Eqn 8-19 */
mu = M / (a * (1.0 - e2 / 4.0 - 3.0 * e4 / 64.0 - 5.0 * e6 / 256.0));

/* Eqn 3-26 */
phi_1 =    mu +
           (3.0 * e1 / 2.0 - 27.0 * e1_3 / 32.0) * sin(2.0 * mu) +
           (21.0 * e1_2 / 16.0 - 55.0 * e1_4 / 32.0) * sin(4.0 * mu) +
           (151.0 * e1_3 / 96.0) * sin(6.0 * mu);

sin_phi_1 = sin(phi_1);
cos_phi_1 = cos(phi_1);
sin2_phi_1 = sin_phi_1 * sin_phi_1;
cos2_phi_1 = cos_phi_1 * cos_phi_1;
tan_phi_1 = tan(phi_1);

/* Eqn 8-21 thru 8-25 */
C1 = e_primed_2 * cos2_phi_1;
C1_2 = C1 * C1;

T1 = tan_phi_1 * tan_phi_1;
T1_2 = T1 * T1;

N1 = a / sqrt(1.0 - e2 * sin2_phi_1);

temp = pow ((1.0 - e2 * sin2_phi_1), 1.5);
R1 = a * (1.0 - e2) / temp;

D  = E / (N1 * scale_factor);
D2 = D * D;
D3 = D2 * D;
D4 = D2 * D2;
D5 = D3 * D2;
D6 = D4 * D2;

/* Eqn 8-17 */
t1 = (5.0 + 3.0 * T1 + 10.0 * C1 - 4.0 * C1_2 - 9.0 * e_primed_2) * D4 / 24.0;
t2 = (61.0 + 90.0 * T1 + 298.0 * C1 + 45.0 * T1_2 - 
      252.0 * e_primed_2 - 3.0 * C1_2) * D6 / 720.0;
*lat = phi_1 - (N1 * tan_phi_1 / R1) * (D2 / 2.0 - t1 + t2);

/* Eqn 8-18 */
t1 = (1.0 + 2.0 * T1 + C1) * D3 / 6.0;
t2 = (5.0 - 2.0 * C1 + 28.0 * T1 - 3.0 * C1_2 + 8.0 * e_primed_2 +
      24.0 * T1_2) * D5 / 120.0;

*lon = lon_origin + (D - t1 + t2) / cos_phi_1;

return TRUE;
}


/*----------------------------------------------------------------------------*/
/* Given the UTM Zone, return the Central meridian in degrees                */
/*----------------------------------------------------------------------------*/
int compute_cm_from_zone(int zone)
{
int offset;               /* number of zones away from zone 30 */
int sgn;                  /* Sign of the offset */

sgn = ((offset = (zone - 30)) <= 0) ? -1 : 1;
if (sgn > 0)
	{
	return (offset * 6 + sgn * 3) - 6;
	}
else
	{
	return offset * 6 + sgn * 3;
	}
}


